@@ -16,11 +16,11 @@
 
   PROGRAMMERS:
 
-    martin.isenburg@rapidlasso.com  -  http://rapidlasso.com
+    info@rapidlasso.de  -  https://rapidlasso.de
 
   COPYRIGHT:
 
-    (c) 2005-2018, martin isenburg, rapidlasso - fast tools to catch reality
+    (c) 2005-2018, rapidlasso GmbH - fast tools to catch reality
 
     This is free software; you can redistribute and/or modify it under the
     terms of the GNU Lesser General Licence as published by the Free Software
@@ -31,6 +31,7 @@
 
   CHANGE HISTORY:
 
+    9 November 2022 -- support of COPC VLR and EVLR
     19 April 2017 -- support for selective decompression for new LAS 1.4 points
     1 February 2017 -- better support for OGC WKT strings in VLRs or EVLRs
     22 June 2016 -- set default of VLR header "reserved" to 0 instead of 0xAABB
@@ -49,7 +50,7 @@
 #ifndef LAS_DEFINITIONS_HPP
 #define LAS_DEFINITIONS_HPP
 
-#define LAS_TOOLS_VERSION 210117
+#define LAS_TOOLS_VERSION 221128
 
 #include <stdio.h>
 #include <string.h>
@@ -84,6 +85,10 @@
 #define LAS_TOOLS_IO_IBUFFER_SIZE   262144
 #define LAS_TOOLS_IO_OBUFFER_SIZE   262144
 
+#ifndef MAX_PATH // linux
+#define MAX_PATH 256
+#endif
+
 class LASvlr
 {
 public:
@@ -154,6 +159,39 @@ private:
   U8 data[26];
 };
 
+class LASvlr_copc_info
+{
+public:
+  F64 center_x;  // Actual (unscaled) X coordinate of center of octree
+  F64 center_y;  // Actual (unscaled) Y coordinate of center of octree
+  F64 center_z;  // Actual (unscaled) Z coordinate of center of octree
+  F64 halfsize;  // Perpendicular distance from the center to any side of the root node.
+  F64 spacing;   // Space between points at the root node. This value is halved at each octree level
+  U64 root_hier_offset; // File offset to the first hierarchy page
+  U64 root_hier_size;   // Size of the first hierarchy page in bytes
+  F64 gpstime_minimum;    // Minimum of GPSTime
+  F64 gpstime_maximum;   // Maximum of GPSTime
+  U64 reserved[11];    // Must be 0
+};
+
+class LAScopc_voxelkey
+{
+public:
+  I32 depth;
+  I32 x;
+  I32 y;
+  I32 z;
+};
+
+class LASvlr_copc_entry
+{
+public:
+  LAScopc_voxelkey key;
+  U64 offset;
+  I32 byte_size;
+  I32 point_count;
+};
+
 class LASheader : public LASquantizer, public LASattributer
 {
 public:
@@ -207,6 +245,11 @@ public:
   LASvlr_classification* vlr_classification;
   LASvlr_wave_packet_descr** vlr_wave_packet_descr;
 
+  // LAZ 1.4 format 6 7 8 with COPC only
+  U32 number_of_copc_entries;
+  LASvlr_copc_info* vlr_copc_info;
+  LASvlr_copc_entry* vlr_copc_entries;
+
   LASzip* laszip;
   LASvlr_lastiling* vlr_lastiling;
   LASvlr_lasoriginal* vlr_lasoriginal;
@@ -327,6 +370,10 @@ public:
       vlr_classification = 0;
       if (vlr_wave_packet_descr) delete [] vlr_wave_packet_descr;
       vlr_wave_packet_descr = 0;
+      if (vlr_copc_info) delete vlr_copc_info;
+      vlr_copc_info = 0;
+      if (vlr_copc_entries) delete [] vlr_copc_entries;
+      vlr_copc_entries = 0;
       number_of_variable_length_records = 0;
     }
   };
@@ -640,11 +687,11 @@ public:
     {
       if (keep_existing)
       {
-        i = number_of_variable_length_records;
+        i = number_of_extended_variable_length_records;
       }
       else
       {
-        for (i = 0; i < number_of_variable_length_records; i++)
+        for (i = 0; i < number_of_extended_variable_length_records; i++)
         {
           if ((strcmp(evlrs[i].user_id, user_id) == 0) && (evlrs[i].record_id == record_id))
           {
@@ -709,13 +756,19 @@ public:
           if (delete_data)
           {
             delete [] evlrs[i].data;
+            evlrs[i].data = 0;
           }
         }
         number_of_extended_variable_length_records--;
         if (number_of_extended_variable_length_records)
         {
-          evlrs[i] = evlrs[number_of_extended_variable_length_records];
-          evlrs = (LASevlr*)realloc(evlrs, sizeof(LASvlr)*number_of_extended_variable_length_records);
+          // Fix for #67. I do not understand why realloc corrupts the memory.
+          LASevlr* tmp = (LASevlr*)calloc(number_of_extended_variable_length_records, sizeof(LASevlr));
+          for (U32 j = 0, k = 0; j < number_of_extended_variable_length_records+1; ++j) { if (j != i) { tmp[k++] = evlrs[j]; }}
+          free(evlrs);
+          evlrs = tmp;
+          //evlrs[i] = evlrs[number_of_extended_variable_length_records];
+          //evlrs = (LASevlr*)realloc(evlrs, sizeof(LASvlr)*number_of_extended_variable_length_records);
         }
         else
         {
@@ -925,6 +978,23 @@ public:
     }
   }
 
+  void del_copc()
+  {
+    if (vlr_copc_info)
+    {
+      remove_vlr("copc", 1);
+	  delete vlr_copc_info;
+      vlr_copc_info = 0;
+    }
+
+    if (vlr_copc_entries)
+    {
+      remove_evlr("copc", 1000);
+      delete[] vlr_copc_entries;
+      vlr_copc_entries = 0;
+    }
+  }
+
   void set_geo_ogc_wkt(const I32 num_geo_ogc_wkt, const CHAR* geo_ogc_wkt, BOOL in_evlr=FALSE)
   {
     I32 null_terminator = 0;
